

# Description of data analysis regarding sex-specific changes in autosomal DNA methylation rate in ageing common terns"

### Manuscript: "Sex-specific changes in autosomal methylation rate in ageing common terns" published in "Frontiers Ecology And Evolution", DOI: 10.3389/fevo.2023.982443
*by Britta S. Meyer, Maria Moiron, Calvinna Caswara, Giulio Formenti, William Chow, Jonathan Wood, Kevin Howe, Oliver Fedrigo, Erich D. Jarvis, Miriam Liedvogel, Sandra Bouwhuis*

**Britta S. Meyer**, britta@bsmeyer.com, [https://orcid.org/0000-0002-2549-1825](https://orcid.org/0000-0002-2549-1825), Max Planck Institute for Evolutionary Biology, Behavioural Genomics; Plön; Germany

**Maria Moiron**, Institute of Avian Research; Wilhelmshaven; Germany

**Miriam Liedvogel**,  Max Planck Institute for Evolutionary Biology, Behavioural Genomics; Plön; Germany & Institute of Avian Research; Wilhelmshaven; Germany

**Sandra Bouwhuis**, Institute of Avian Research; Wilhelmshaven; Germany



written by Britta Meyer in December 2022


In the manuscript "*Sex-specific changes in autosomal methylation rate in ageing common terns*" published in Frontiers Ecology And Evolution, we investigate sex- and age-specific changes in autosomal methylation of common terns, relatively long-lived migratory seabirds. We collected blood at 1-, 3- and/or 4-year intervals, extracted DNA from the erythrocytes and estimated autosomal DNA methylation by mapping Reduced Representative Bisulfite Sequencing reads to a *de novo* assembled reference genome. This document combines the various scripts used for the analyses.

Note:
For more information regarding further (meta) data or analyses, please contact sandra.bouwhuis@ifv-vogelwarte.de or britta@bsmeyer.com. Please also contact sandra.bouwhuis@ifv-vogelwarte.de if you'd like to use the shared data. These data are part of those collected in an ongoing individual-based longitudinal population study and additional data are likely to be available, and we are generally happy to collaborate, provided that none of our own staff or students is working on a conflicting project. 

#### 1. Reduced representation bisulfite sequencing (RRBS) data analysis

The raw RRBS reads can be found at: https://www.ebi.ac.uk/ena, PRJEB48910.  

##### 1.1 Preparing and trimming the reads

Ubam files were generated by the CeMM Research Center for Molecular Medicine, Vienna, Austria, which we further processed with the software package  *RefFreeDMA*, (Klughammer et al. 2015). After conversion of unmapped bam files to fastq with SamToFastq.jar from *picard v1.118*, the quality of the sequencing reads was checked using *Fastqc v0.11.5* and *Multiqc v1.8*. Spiked-in sequences were used to estimate over- and underconversion using the summary parsing script by [*RefFreeDMA*](https://github.com/jklughammer/RefFreeDMA/blob/858807127682c075f4c7dc150b2b8469a09af2b1/scripts/parse_stats.sh) (Klughammer et al. 2015). 

##### 1.1.1 Converting from ubam to fastq
This was done using the function picard_sam_to_fastq - with settings kindly provided by [*RefFreeDMA*](https://github.com/jklughammer/RefFreeDMA/blob/8cfcf83464e6d25e0f7c29e22bb6d7679c2801f3/scripts/prepareReads.sh) (Klughammer et al. 2015).

```bash
working_dir=$1
bam_dir=$working_dir/unmapped_bam
picard_path=$4
trim_galore_path=$5
cutadapt_path=$6

picard_sam_to_fastq { 
    input_bam=$1 |
    output_fastq=$2 |
    java -Xmx2g -jar $picard_path/SamToFastq.jar I=$input_bam F=$output_fastq INCLUDE_NON_PF_READS=false |
}
```

##### 1.1.2 Trimming the reads
*Trimgalore v0.3.3* (Krueger et al., 2021) was used to remove adaptors to guarantee good mapping and to remove low quality bases (≥20) as well as short read fragments (≥ 16 bp) - with settings kindly provided by [*RefFreeDMA*](https://github.com/jklughammer/RefFreeDMA/blob/8cfcf83464e6d25e0f7c29e22bb6d7679c2801f3/scripts/prepareReads.sh) (Klughammer et al. 2015).

```bash
function trim_galore {
        input_fastq=$1
        output_dir=$2
        #trimGalore parameters
        #Adapter
        a="AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC"
        #quality trimming
        q=20
        #stringency: Overlap with adapter sequence required to trim a sequence
        s=1
        #Maximum allowed error rate
        e=0.1
        #Minimum Read length
        l=16
        echo $q $s $e $l
        $trim_galore_path/trim_galore -q $q --phred33 -a $a --stringency $s -e $e --length $l --output_dir $output_dir $input_fastq
}
```

##### 1.2 Running bismark

We mapped our bisulfite reads to the newly generated reference genome SteHir1 (submitted to NCBI as GCA_009819605.1, as part of the Vertebrate Genomes Project [VGP](https://genomeark.github.io/genomeark-all/Sterna_hirundo/).

##### 1.2.1 Running `bismark_genome_preparation`
We *in-silico* bisulfite converted and indexed this genome. These two steps can be done with  [*Bismark*](https://github.com/FelixKrueger/Bismark/tree/master/Docs). The script for genome preparation was named (/SOMEPATH/scripts/bismark_genome_prep.sh),  the code is:

```bash
#prepare the tern_genome with bismark
ref='/SOMEPATH/references/VGP'
bowtie2='/SOMEPATH/bowtie2'
bismark_folder='/SOMEPATH/bin'
working_dir='/SOMEPATH/Bismark/genome_prep'

#go to the working directory
cd $working_dir
set -o pipefail

#start bismark command
#bismark_genome_preparation [options] <path_to_genome_folder>
$bismark_folder/bismark_genome_preparation --bowtie2  --verbose --genomic_composition  $ref

```


##### 1.2.2 Running  `bismark`

In the second step the RRBS reads are mapped to the *in-silico* bisulfite converted genome (generated through the step above).

We used *Bismark v0.22.3* (Krueger & Andrews 2011) to prepare and index the reference genome for subsequent mapping of the bisulfite-converted reads using *Bowtie 2 v2.4.1* (Langmead & Salzberg 2012), allowing for one mismatch (--score_min L,0,-0.20). 

The common tern RRBS reads are 50 bp short and they were sequenced as Single-End: 1x50 bp. To decide the maximum alignment penalty and allowed mismatches we calculated as follows:

Maximum alignment penalty examples: 

-0.2 (L,0,-0.2): 50 bp * -0.2 = -10  
-0.6 (L,0,-0.6): 50 bp * -0.6 = -30

Since we score each mismatch as -6, this would mean a maximum number of allowed mismatches of: 

-0.2: -10 / 6 = 1 mismatch  
-0.6: -30 / 6 = 5 mismatches

If we decide for -0.2, we allow 1 mismatch, to map a read to the genome.

```perl
#!/usr/bin/perl -w
# use as:  perl XX.pl 

use strict;
my $qsub_template = do { local $/; <DATA> };
my @files = qw/ HERE NAMES OF ALL TRIMMED FASTQ FILES SEPERATED BY SPACE, e.g. VD922_S34209_trimmed.fq VD945_S34213_trimmed.fq /;

for my $file ( @files ) {

    (my $sbatch_cmd = $qsub_template) =~ s/!file!/$file/g;
    my $sbatch_out = "$file.bismark.sh";
    open my $OUT, ">", $sbatch_out or die "Could not open file $sbatch_out: $!";
    print $OUT $sbatch_cmd;
    close $OUT;
    system "sbatch < $sbatch_out";
} 

__DATA__
#!/bin/bash
#  specify the job name
#SBATCH --job-name=bismark
#  how many cpus are requested
#SBATCH --ntasks=8
#  run on one node
#SBATCH --nodes=1
#  maximum walltime,
#SBATCH --time=XX:XX:XX
#  maximum requested memory
#SBATCH --mem=XXG
#SBATCH --partition=standard

#prepare the tern_genome with bismark
ref='/SOMEPATH/references/VGP/'
bsref='/SOMEPATH/references/VGP/'

bismark_folder='/bin'
working_dir='/SOMEPATH/Bismark/bismark_result'

#results from RefFreeDMA, run named "RefFreeDMA_newdeco_newref"
fastq="/SOMEPATH/RefFreeDMA_newdeco_newref/fastq/"

#load modules
module load python/3.7.1

#go to working directory
cd $working_dir
set -o pipefail

#start bismark command
$bismark_folder/bismark --bowtie2 -q --phred33-quals --score_min L,0,-0.20 --genome $bsref -se $fastq/!file! -o $working_dir/btw2_0.20_trim


```
##### 1.3  Running *FASTQC* on the fastq from "RefFreeDMA_newdeco_newref" trimming

Code used for *FASTQC*:

```bash
#!/usr/bin/perl -w
# use as:  perl XX.pl 
use strict;


my $qsub_template = do { local $/; <DATA> };

my @files = qw/HERE NAMES OF ALL TRIMMED FASTQ FILES SEPERATED BY SPACE, e.g. VD922_S34209_trimmed.fq VD945_S34213_trimmed.fq /;

for my $file ( @files ) {

    (my $sbatch_cmd = $qsub_template) =~ s/!file!/$file/g;
    my $sbatch_out = "$file.fastqc.sh";
    open my $OUT, ">", $sbatch_out or die "Could not open file $sbatch_out: $!";

    print $OUT $sbatch_cmd;

    close $OUT;
    system "sbatch < $sbatch_out";
    ;
} 

__DATA__
#!/bin/bash
##  specify the job name
#SBATCH --job-name= Fastqc_RRBS
#  how many cpus are requested
#SBATCH --ntasks= 10
#  run on one node
#SBATCH --nodes=1
#  maximum walltime,
#SBATCH --time=XX:XX:XX
#  maximum requested memory
#SBATCH --mem=XXG
#SBATCH --partition=standard

#change to working directory
cd RefFreeDMA_experiments/RefFreeDMA_newdeco_newref/fastq 
#load modules
module load java/x64/8u121
#run fastqc
fastqc !file! --outdir=/SOMEPATH/05multiqc_RRBS/fastqc -t 6
```


##### 1.4  Running `bismark_methylation_extractor` with ignore function to handle the Mbias

Methylation extraction was conducted with *Bismark Extractor v0.22.3* (Krueger & Andrews 2011) with the ‘*-ignore*’ option to remove unmethylated cytosines introduced during the end-repair step. With this command we can ignore the last 2 bases at the 3' end . Details can be found on [Felix Krüger's website](https://www.bioinformatics.babraham.ac.uk/projects/bismark/RRBS_Guide.pdf).


Here the code with the ‘*-ignore*’ function:

```perl
#!/usr/bin/perl -w
## use as:  perl XX.pl 
use strict;

my $qsub_template = do { local $/; <DATA> };

my @files = qw/ HERE NAMES OF ALL TRIMMED FASTQ FILES SEPERATED BY SPACE, e.g. VD17891_S37496_trimmed_bismark_bt2.bam VD17888_S37499_trimmed_bismark_bt2.bam /;

for my $file ( @files ) {

    (my $sbatch_cmd = $qsub_template) =~ s/!file!/$file/g;
    my $sbatch_out = "$file.bismark.ignore.sh";
    open my $OUT, ">", $sbatch_out or die "Could not open file $sbatch_out: $!";
    print $OUT $sbatch_cmd;
    close $OUT;
    system "sbatch < $sbatch_out";
} 

__DATA__
#!/bin/bash
##  specify the job name
#SBATCH --job-name= bismark_ign
#  how many cpus are requested
#SBATCH --ntasks=8
#  run on one node
#SBATCH --nodes=1
#  maximum walltime,
#SBATCH --time=XX:XX:XX
#  maximum requested memory
#SBATCH --mem=XXG
#SBATCH --partition=standard

#prepare environment
ref='/SOMEPATH/references/VGP/'
bsref='/SOMEPATH/references/VGP/'

bismark_folder='/SOMEPATH/bin'
working_dir='/SOMEPATH/Bismark/bismark_result'

RRBS_mapped='/SOMEPATH/Bismark/bismark_result/btw2_0.20_trim'
results='/SOMEPATH/Bismark/bismark_result/result_ignore_extr'

bowtie2='/SOMEPATH/bowtie2'
samtools='/SOMEPATH/samtools' #Version: 1.9


#load modules
module load python/3.7.1

#go to working directory
cd $working_dir
set -o pipefail

#with ignore option
$bismark_folder/bismark_methylation_extractor -s --bedGraph --merge_non_CpG --gzip --multicore 10 --ignore_3prime 2 --genome_folder $bsref $RRBS_mapped/!file! --cytosine_report

```

##### 1.5 Quality of methylation extraction

For getting the reports about the methylation extraction with *Multiqc v1.8* we ran the commands below:

```
cp /SOMEPATH/Bismark/bismark_result/btw2_0.20_trim/*SE_report.txt /SOMEPATH/Bismark/bismark_result/

$bismark_folder/bismark2report 

module load python/3.6.0
cd /SOMEPATH//05multiqc_RRBS 
multiqc /SOMEPATH/Bismark/bismark_result/btw2_0.20_trim /SOMEPATH/RefFreeDMA_newdeco_newref/fastq /SOMEPATH/Bismark/bismark_result/ -o fastq_bismark_v1 -n fastq_bismark_v1 -f
```

##### 1.6 Merging neighbouring cytosines

The bismark command above extracted context-dependent (CpG/CHG/CHH) methylation and produced three methylation and other (optional) output files:
 
- `CpG_context_test_dataset_bismark_bt2.txt.gz`
- `CHG_context_test_dataset_bismark_bt2.txt.gz`
- `CHH_context_test_dataset_bismark_bt2.txt.gz`
-  a bedGraph
-  a Bismark coverage file
-  and a cytosine_report.  

The cytosines in the coverage files loaded into methylkit are normally considered individually, i.e., separately from both reads in each CpG site. But based on the notion that methylation occurs symmetrically, and to increase the coverage of some of our CpGs, we merged adjacent cytosines of both reads per CpG site.


###### Using merge_CpG.py

The script is called "[merge_CpG.py](https://github.com/rcristofari/penguin-tools/blob/master/merge_CpG.py)" and written by Robin Cristofari. It merges CpG values of neighbouring CpGs from the cytosine report by Bismark and creates a new "fake" bismark output, which gets loaded into Methylkit.

Here is a copy of the perl script "multiple_merge.pl", with which we had submitted several jobs to the Cluster in parallel:

```{bash}
#!/usr/bin/perl -w
use strict;

my $qsub_template = do { local $/; <DATA> };
my @files = qw/  HERE NAMES OF ALL TRIMMED FASTQ FILES SEPERATED BY SPACE, e.g. BSF_0414_HNM5LBBXX_4__VD904_S34204_trimmed_bismark_bt2.CpG BSF_0414_HNM5LBBXX_4__VD900_S34203_trimmed_bismark_bt2.CpG /;


for my $file ( @files ) {
  
  (my $qsub_cmd = $qsub_template) =~ s/!file!/$file/g;
  my $qsub_out = "$file.merge_robin.sh";
  open my $OUT, ">", $qsub_out or die "Could not open file $qsub_out: $!";
  
  print $OUT $qsub_cmd;
  close $OUT;  
  system "sbatch < $qsub_out";
} 

__DATA__
#!/bin/bash
##  specify the job name
#SBATCH --job-name=merge_robin
#  how many cpus are requested
#SBATCH --ntasks=1
#  run on one node
#SBATCH --nodes=1
#  maximum walltime, here 10min
#SBATCH --time=XX:XX:XX
#  maximum requested memory
#SBATCH --mem=XXG
#  write std out and std error to these files
#SBATCH --error=/SOMEPATH/stdout/merge_robin.%j.err
#SBATCH --output=/SOMEPATH/stdout/merge_robin.%j.out
#  send a mail for job start, end, fail, etc.
#SBATCH --mail-type=FAIL
#SBATCH --partition=standard

module load python/3.7.1
cd /SOMEPATH/Bismark/bismark_result/result_ignore_extr/
python /SOMEPATH/scripts_server/merge_CpG5.py --input !file!_report.txt.gz
gzip !file!_merged.cov 
```

##### 1.7 Loading meta data to R

```
library(methylKit)

setwd("/SOMEPATH/meta_RRBS/")
terns_data = read.csv(file="/SOMEPATH/meta_RRBS/Epigenetics_13-14-17_updated_gender.txt", header=T, sep="\t") # load metadata 
terns_data =terns_data[,1:23] # get the names of raw files 
raw_CpG <- read.csv('/SOMEPATH/meta_RRBS/raw_bam_filesnames.txt',sep="\t", header=F) 
ID=as.data.frame(str_split_fixed(raw_CpG$V1,"[_.]",8)) 
raw_CpG$Sample_Name =paste(ID[,5],"_",ID[,6])
raw_CpG$Sample_Name= gsub(" ", "", raw_CpG$Sample_Name, fixed = TRUE)
raw_CpG$lane=paste(ID[,1],"_",ID[,2],"_",ID[,3],"_",ID[,4])
raw_CpG$lane= gsub(" ", "", raw_CpG$lane, fixed = TRUE)
raw_CpG$sample=paste(ID[,5])
raw_CpG$sample_ID =paste(ID[,6])
raw_CpG$cpg_merge =paste0(ID[,1],"_",ID[,2],"_",ID[,3],"_",ID[,4],"__",ID[,5],"_",ID[,6],"_trimmed_bismark_bt2.CpG_merged.cov.gz") 
raw_CpG= raw_CpG  %>% dplyr::select(V1,Sample_Name,lane,sample,sample_ID,cpg_merge)%>%filter(!(lane == "BSF_0426_HTHTJBBXX_2")) #exclude
merged_terns_data_CpG = merge(raw_CpG,terns_data, by="sample")
PCR = read.table(file="/SOMEPATH/meta_RRBS/PCRcycles_CEMM.txt", header=T, sep="\t")
colnames(PCR) = c("runID","sample","cycles") 
#merge with the former table
meta = merge(PCR,merged_terns_data_CpG, by="sample")
```
##### 1.8  Loading the artificially generated bismark files to Methylkit

We used the R-package [Methylkit](https://github.com/al2na/methylKit) to create a methylation matrix of positions, which had at least 10x coverage. CpGs with a coverage >99.9th percentile most likely result from PCR bias and were removed. These two steps were performed using the *filterByCoverage* function in *methylKit*. 
Methylation call distributions between samples were normalized using the *normalizeCoverage* function to reduce the potential bias of systematic oversampling in some samples. We then merged CpG positions of the different samples using the unite function to make sure positions were sequenced in at least 70 % of all samples (within and across the sexes) to facilitate a longitudinal analysis approach.

```
library(methylKit)
setwd("/SOMEPATH/Bismark/bismark_result/result_ignore_extr/") 
name_cov2= as.character(meta$cpg_merge)
file_labels <- as.list(name_cov2)
sample_id <- as.list(meta$Sample_Name)
meta$treatment= (1)

treatment_list =as.list(meta$treatment)
setwd("/SOMEPATH/Bismark/bismark_result/result_ignore_extr/")
big_DB= methRead(file_labels, 
                  sample.id=as.list(sample_id), 
                  assembly = "SteHir", 
                  mincov = 1, 
                  pipeline = 'bismarkCoverage', 
                  context = "CpG", 
                  treatment = as.numeric(treatment_list),
                  dbtype = "tabix",
                  dbdir = "methylDB"
                  ) 
```
###### 1.8.1 Filtering the methylation reads low (10x) and high coverage (99.9 percentile)
```
filtered.big_DB=filterByCoverage(big_DB,lo.count=10,lo.perc=NULL,
                                hi.count=NULL,hi.perc=99.9)
```
###### 1.8.2  Normalize methylation reads
```
#normalize by mean coverage
filtered.norm_big_DB = normalizeCoverage(filtered.big_DB,method="mean", save.db=TRUE,dbdir="methylDB")
```
###### 1.8.3 Reduce data to adults sampled
```
meta_adult_bug =meta[meta$status == 'adult',]
treatment_sex <- (meta_adult_bug$sex)
sample.ids_bug=meta_adult_bug$Sample_Name
filtered.norm_big_DB_sex=reorganize(filtered.norm_big_DB,sample.ids=sample.ids_bug,treatment=treatment_sex)
```
###### 1.8.4 Create a unite file with 70 % of sharedness
Here we create an object within *methylKit* using the *-unite* function, which includes positions which occur in at least 70 % of the samples (within and across the two sexes). We set "min.per.group = 26L" as 25.9 samples are 70 % of 37 samples. This results in a methylBaseDB object with 32951 rows.


```
unite_terns=methylKit::unite(filtered.norm_big_DB_sex,destrand=FALSE,min.per.group = 26L)

```

Afterwards we use the getData function from *methylKit* Objects to retrieve the methylation information for each position.

```
mat=getData(unite_terns)

```
###### 1.8.5 Remove sites with low variation 

Following Meng et al. (2010) and Sziráki et al. (2018), sites that showed little or no variation were removed, applying a threshold of a standard deviation <0.15. 

```
#1. for GLMM with positions
meth.mat = mat[, unite_terns@numCs.index]/
  (mat[,unite_terns@numCs.index] + mat[,unite_terns@numTs.index] )
names(meth.mat)=unite_terns@sample.ids
summary(unite_terns@sample.ids) #74
unite_terns@sample.ids
start=mat$start
chromosome=mat$chr

meth.mat.con=cbind(chromosome, start,meth.mat) 

#2. add position column with string, e.g. like scaffold_112_arrow_ctg1_28484
meth.mat.con$pos <- paste(meth.mat.con$chromosome,meth.mat.con$start, sep = "_")

#3. Get sites with low variation
sds=matrixStats::rowSds(as.matrix(meth.mat.con[3:74]),na.rm=TRUE)
sd.threshold0.52=0.52
cutoff0.52_gr=stats::quantile(sds,sd.threshold0.52,na.rm=TRUE)
meth.mat.con_sds0.52=meth.mat.con[sds>cutoff0.52_gr,]; 
combined_index = unite_terns@numCs.index 
combined_index= append(combined_index,unite_terns@numTs.index)
combined_index=sort(combined_index)
combined_index=as.vector(combined_index)
GLMMmat = dplyr::select(mat,combined_index)

#4. get position
start=mat$start
chromosome=mat$chr
GLMMmat.con=cbind(chromosome, start,GLMMmat)

##5. create a column with fused chromosome and start information
GLMMmat.con$pos <- paste(GLMMmat.con$chromosome,GLMMmat.con$start, sep = "_") 

#6. set position as rownames
rownames(GLMMmat.con) <-GLMMmat.con$pos 

#positions to keep from meth.matrix removed low sds sites
meth.mat.con_sds0.52=meth.mat.con_sds0.52$pos

#7. reduce dataframe 
GLMMmat.con0.52_subset =GLMMmat.con[which(rownames(GLMMmat.con) %in% meth.mat.con_sds0.52), ]
```
###### 1.8.6 Reformat the methylation matrix for the GLMM

In order to proceed with the methylation matrix (where positions are represented as rows and individuals as columns), we need to transpose (reshape) the data for the GLMM.

```
library(reshape2)
tab2 <- reshape2::melt(GLMM.con0.52_subset[,-1:-2], id.vars="pos")

#remove Ts and get Ind info
tab2$CT <- str_remove_all(as.character(tab2$variable), "[nums0123456789]") 
tab2$Ind <- str_replace(as.character(tab2$variable), "numTs","") %>% str_replace(., "numCs","")
Split <- split(tab2[,c("pos","value","Ind")], tab2$CT, drop=TRUE) # split tables into list entrys
mydf <- do.call(cbind, Split) #bind them together again

#now you will have two times the start info and the Ind info, to check that they match (so you can remove 1):
true=mydf$C.pos == mydf$T.pos #all TRUE
true= mydf$C.Ind == mydf$T.Ind #all TRUE
summary(true)
df_positions <- mydf[,c("C.pos","C.Ind","C.value","T.value")]
colnames(df_positions) <- c("pos","Ind","C","T")
summary(unite_terns@sample.ids) #74
old_val = c(1:74)
new_val = c(unite_terns@sample.ids)
lookup <- data.frame(old_val = c(1:74), 
                     new_val = c(unite_terns@sample.ids))
library('data.table')
setDT(df_positions)
df_positions[, Ind := factor(Ind, levels = old_val, labels = new_val)]
write.table(df_positions,file="/SOMEPATH/meta_data/NEW_data_10x70_pergr_GLMM_74adults_sd_0.52.txt" )


#load meta data 
meta2=read.csv(file="/SOMEPATH/Rdata_files_cluster/meta2.txt",sep = ",") # information about Sample_Name, average_age, delta_age, sex,year, birdID, age_at_sampling
df_positions=read.csv(file="/SOMEPATH/meta_data/NEW_data_10x70_pergr_GLMM_74adults_sd_0.52.txt",sep = " ")
names(df_positions)[names(df_positions) == "Ind"] <- "Sample_Name"
meta2$year <- as.factor(meta2$year)
meta2$sex <- as.factor(meta2$sex)
meta2=dplyr::select(meta2, Sample_Name, average_age, delta_age, sex,year, birdID, age_at_sampling)

#prepare for GLMM
positions_meta2_inputGLMM=merge(df_positions, meta2, by="Sample_Name") 
positions_meta2_inputGLMM$trials =positions_meta2_inputGLMM$C+positions_meta2_inputGLMM$T
positions_meta2_inputGLMM$year <- as.factor(positions_meta2_inputGLMM$year)
positions_meta2_inputGLMM$sex <- as.factor(positions_meta2_inputGLMM$sex) 
positions_meta2_inputGLMM$obs_effect = (1:nrow(positions_meta2_inputGLMM))
plyr::count(positions_meta2_inputGLMM$birdID)
write.table(positions_meta2_inputGLMM,file="/SOMEPATH/meta_data/newdf_10x_70_74_new_sds0.52_gr_adult_bug.txt")
```

#### 2. GLMM
To identify and partition sources of variation in autosomal methylation rates, we used the R package glmmTMB (Brooks et al. 2017): we ran a generalized linear mixed model (GLMM) with the BFGS algorithm as an optimizer, using the number of methylated and unmethylated Cs (represented by the sequenced Cs and Ts) for each position as our dependent variable, assuming a binomial error distribution (Lea et al. 2017). As fixed effects we added ‘sex’ (as a two-level class variable using males as the reference category) and age (as a covariate). With respect to the latter, each individual’s age was partitioned into an ‘average age’ and ‘delta age’ component following van de Pol & Wright (2009). Average age was calculated as the average of all ages at which we assessed a bird’s autosomal methylation rate, while delta age was calculated as the difference between the bird’s actual age and its average age (i.e. delta age = age – average age). When adding both age variables as covariates, average age represents the among-individual, and delta age the within-individual age effect (van de Pol & Wright 2009). 

##### 2.1. Loading the data and removing missing data
Here we load the matrix of all positions with a minimum coverage of 10x and shared by 70 % of the samples.
We remove positions with missing data.

```
positions_meta2_inputGLMM =read.csv(file = "/SOMEPATH/meta_data/newdf_10x_70_74_new_sds0.52_gr_adult_bug.txt",sep = "\t")
positions_meta2_inputGLMM_na = na.omit(positions_meta2_inputGLMM)
write.table(positions_meta2_inputGLMM_na,file="/SOMEPATH/meta_data/newdf_10x_70_74_new_sds0.52_gr_na_adult_bug.txt")

```
##### 2.2 Removing the CpG positions of the sex-chromosomes

```{r removing both chromosome Z and W}
#create an object without sex chromosomes - exclude both Z and W
positions_meta2_inputGLMM_na_noW_Z=positions_meta2_inputGLMM_na[!grepl("Super_scaffold_4|Super_scaffold_16",positions_meta2_inputGLMM_na$pos),]

write_csv(positions_meta2_inputGLMM_na_noW_Z,file="/SOMEPATH/meta_data/newdf_10x_70_74_new_sds0.52_gr_na_noW_Z.txt") # named GLMM_dataframe.txt in the repository
```

##### 2.3 Run GLMM model

Here we run the full model using the r-package "glmmTMB". 

- bird identity (N=34), observation (N= 927490), and position identity (N=15700) as random effects
- sex, average age and delta age as fixed effects
- interaction between sex and delta_age
- We further used the optimizer "BFGS". 


```
library(glmmTMB)
library(methylKit)
library(TMB)
library("gtsummary")
library("parameters")
library("nnet")
library("survival")
library("sjPlot")
library("insight")
library("tidyverse")	
library(performance)

#load data
positions_meta2_inputGLMM_na_noW_Z = read.csv(file="/SOMEPATH/meta_data/newdf_10x_70_74_new_sds0.52_gr_na_noW_Z.txt")  # named GLMM_dataframe.txt in the repository
positions_meta2_inputGLMM_na_noW_Z$obs_effect = (1:nrow(positions_meta2_inputGLMM_na_noW_Z)) # add observation counter

#run the model 
my.model <- glmmTMB::glmmTMB(cbind(C,T) 
                                                                        ~sex*delta_age+average_age +(1|birdID)+(1|obs_effect)+(1|pos),
                                                                        family=binomial,data=positions_meta2_inputGLMM_na_noW_Z,
                                                                        control=glmmTMBControl(optimizer=optim,
                                                                                               optArgs=list(method="BFGS")))
summary(my.model) 
save(my.model,file="~/SOMEPATH/Rdata_files_cluster/m11_newdf_10x_70_74_new_sds0.52_gr_na_noW_Z_glmmTMB.Rdata")
```
##### 2.4 Check for overdispersion
```{r}
check_overdispersion(my.model)
```


#### 3. Visualization 

##### 3.1 Plotting Figure 1

```
library(glmmTMB)
load("~/SOMEPATH/Rdata_files_cluster/m11_newdf_10x_70_74_new_sds0.52_gr_na_noW_Z_glmmTMB.Rdata")
positions_meta2_inputGLMM_na_noW_Z=read.table(file="/SOMEPATH/meta_data/newdf_10x_70_74_new_sds0.52_gr_na_noW_Z.txt", header=T, sep=",")
new_data=positions_meta2_inputGLMM_na_noW_Z # data 

#1. leveling out the average_age --> this only has 1 value now
new_data$average_age = mean(new_data$average_age) #10.92215
#2. create predictions and their residuals for each position 
predictions_m11=predict(my.model,newdata=new_data)
new_data$pred =predictions_m11
new_data$resid=resid(my.model)
#3. create the average of the predictions per sample (aka Sample_Name)  
aggre_pred=aggregate(pred~Sample_Name,new_data,mean) 
#4. For plotting and getting meta data and reduce dataframe 
new_data_subset = unique(new_data[,c("Sample_Name","average_age","delta_age","sex","year","birdID","age_at_sampling")])

#5. Merge predictions with meta data
merge_agg_newd=merge(aggre_pred,new_data_subset,by="Sample_Name")
merge_agg_newd %>%                               # Summary by group using dplyr
  group_by(sex) %>% 
  summarize(min = min(pred),
            q1 = quantile(pred, 0.25),
            median = median(pred),
            mean = mean(pred),
            q3 = quantile(pred, 0.75),
            max = max(pred))

fig = ggplot(merge_agg_newd)+
  geom_point(aes(x=delta_age,y=pred,col=factor(sex),shape=factor(sex)))+
  geom_line(aes(x=delta_age,y=pred,group=birdID,col=factor(sex),linetype=factor(sex))) +
  theme_bw() +
  theme_set(theme_get() + theme(text = element_text(family = 'Arial',size = 14)))+
  scale_color_manual(name = "Sex",
                     values = c('black', 'black'),
                     labels = c("male","female")) +
  scale_linetype_manual(name = "Sex", 
                        values = c('solid', 'dotted'),
                        labels = c("male","female"))+
  scale_shape_manual(name = "Sex",
                     values=c(16, 1),
                     labels = c("male","female"))


set_theme(
  base = theme_bw(),
  legend.size = .8,
  legend.title.size = 0.8,
)
fig= fig + scale_x_continuous(name="delta age (in years)") +
  scale_y_continuous(name="predicted average methylation rate") # Change x and y axis labels, and limits
fig
ggsave("~/SOMEPATH/FINALSUBMISSION/Figures/Fig1.jpg",
  fig,
  width = 7,
  height = 5,
  dpi = 1200
)
```

##### 3.2 Plotting Supplementary Figures

###### 3.2.1 Supplementary Figure 1

Here we show that the autosomal DNA methylation rate (estimated across 15,700 genomic positions) declines with age within individual female (dotted line), but not male (solid line), common terns. Lines represent the model-predicted within-individual change in methylation rate with age, grey areas the 83 % confidence interval around this change.

```
load("~/SOMEPATH/Rdata_files_cluster/m11_newdf_10x_70_74_new_sds0.52_gr_na_noW_Z_glmmTMB.Rdata")

#final plot
p =plot_model(my.model,
              title = "",
              axis.title = c("delta age (in years)","methylation rate (average ± 83 % confidence interval)"), 
              legend.title = "sex",
              # legend.labels= c("male","female"), 
              type = "pred", 
              terms = c("delta_age", "sex"),
              colors = "bw", ci.lvl=0.83
)
set_theme(
  base = theme_bw(),
  legend.size = .8,
  legend.title.size = 0.8,
  
)
p = p + 
  scale_linetype_manual(values = c("solid","dotted"), labels = c("male","female"))+
theme_set(theme_get() + theme(text = element_text(family = 'Arial')))
p
#sjPlot::save_plot(filename="~/SOMEPATH/figures/final_m11_newdf_10x_70_74_new_sds0.52_gr_na_noW_Z_glmmTMB_switched_interaction1_cI_.png")
ggsave("~/SOMEPATH/FINALSUBMISSION/Suppl_material/S1.jpg",
       p,
       width = 7,
       height = 5,
       dpi = 1200
)
```

###### 3.2.2 Supplementary Figure 2

Here we create violin plots showing the mean autosomal methylation rates (across 15,700 genomic positions) for male and female common terns.

```
g <- ggplot(newdf_10x_70_74_new_sds0.52_gr_na_noW_Z, aes((C/(C+T)),sex))
g + geom_violin() + 
  labs(x="methylation rate",
       y="sex")+ 
  stat_summary(fun = "mean",
               geom = "crossbar", 
               width = 0.7,
               colour = "black")+
  theme_set(theme_get() + theme(text = element_text(family = 'Arial')))+
  scale_y_discrete(labels=c('male', 'female'))

sjPlot::save_plot(filename="~/SOMEPATH/supplementary_tables_figures/S2_newdf_10x_70_74_new_sds0.52_gr_na_noW_Z_violinplot_sex.png")

```

###### 3.2.3 Supplementary Figure 3

Here we show the proportion (in permille) of 15,816 CpGs per chromosome length sequenced in 74 samples obtained from 34 individual common terns across the chromosomes. Smaller scaffolds for which the chromosome was unknown are merged and labelled ‘chrUn’.


```
#count occurences per chromosome 
chromosome_occur=(as_data_frame(GLMM.con0.52_subset))
chromosome_occur =chromosome_occur %>% group_by(chromosome) #group by 94 chromosome
chromosome_occur_count = chromosome_occur %>% summarize(n=n())

#fuse chromosome_unkown to one category, first add column "yes" if "scaffold_" is part of the name in contrast to "Super_scaffold" as these are the chromosomes
chromosome_occur_count =chromosome_occur_count %>%  
  mutate(chromosome_unkown = case_when(grepl("^scaffold_", chromosome) ~ "yes",
                           grepl("^Super_scaffold", chromosome, ignore.case = TRUE) ~"no"))
#split table depending on the value in column chromosome_unkown --> yes or no.
X<-split(chromosome_occur_count, chromosome_occur_count$chromosome_unkown)

#There are 1436 occurences in chromosome unknowns and we will fuse them to one new row
chromosome_occur_count_Yes=sum(X$yes$n) #1436
chromosome_occur_count_Yes =rbind(X$no,c("chromosome_unkown",sum(X$yes$n),"yes"))
chromosome_occur_count_Yes =chromosome_occur_count_Yes %>% dplyr::select(chromosome,n)

#Get the chromosome lengths from https://ftp.ncbi.nlm.nih.gov/genomes/all/GCA/009/819/605/GCA_009819605.1_bSteHir1.pri/GCA_009819605.1_bSteHir1.pri_assembly_report.txt
Chromosome_length =read_tsv(file="~/meta/Chromosomes_length.txt")
Chromosome_length_na = aggregate(Chromosome_length$`Sequence-Length`, by=list(Category=Chromosome_length$`Assigned-Molecule`), FUN=sum)

m1 = merge(Chromosome_length,chromosome_occur_count_Yes, by.x="Sequence-Name",by.y="chromosome", sort =F)
m1 =m1 %>% dplyr::select("Sequence-Name",n,"Sequence-Length","Assigned-Molecule")
final =rbind(m1,c("chromosome_unkown",sum(X$yes$n),"9151577","chromosome_unkown"))
final[final == "chromosome_unkown"] <- "chrUn"

final <- as_tibble(final) # so it prints a little nicer
final=rename(final, c("CpGs positions [bp]" = n,  "Chromosome length [bp]"= "Sequence-Length", "Chromosome"= "Assigned-Molecule"))
final =final %>% dplyr::select("Chromosome","CpGs positions [bp]","Chromosome length [bp]")

#Convert columns to numeric 
final$`CpGs positions [bp]`=as.numeric(final$`CpGs positions [bp]`)
final$`Chromosome length [bp]`=as.numeric(final$`Chromosome length [bp]`)
final=final %>% dplyr::mutate("Ratio [‰]" = (final$`CpGs positions [bp]`/final$`Chromosome length [bp]`)*1000)
final=final %>%dplyr::mutate_if(is.numeric, round, 4)

#plotting
p1= ggplot(final, aes(x=`Chromosome`)) +
  geom_col(aes(y=as.numeric(`Ratio [‰]`))) +
  ylab(" proportion of CpGs per chromosome length [‰]")+
  scale_x_discrete(limits=final$Chromosome, name="chromosome")
p1=p1 + theme(axis.text.x = element_text(angle = 25, hjust=1),family='Arial',base_size = 14)+
  scale_colour_grey() + 
  theme_bw()
p1

ggsave("~/supplementary_tables_figures/Ratio_CpGs_chromosome_Length_barplot15816.jpg",
  p1,
  width = 7,
  height = 5,
  dpi = 1200
)
```


